;==============================================================================
; HP 95LX PrintScreen Utility TSR
; Copyright 1991 Hewlett-Packard Company
; All rights reserved
; Author: Everett Kaser
; Begun: 5-20-91
; Release 1.0: 5-24-91
;==============================================================================

	include	settings.mac

;==============================================================================

cseg	segment	para public 'code'
	assume	cs:cseg, ds:cseg

	org	100h
start:
	jmp	init

;==============================================================================
;
; From the SETUP.ENV file:
;
; baud:	0	 300
;	1	1200
;	2	2400
;	3	4800
;	4	9600
;	5      19200
;
; printer_type:
;	0	Epson FX80
;	1	HP LaserJet
;	2	IBM Proprinter
;
; interface:
;	0	com1
;	1	com2
;	2	IR
;
baud		dw	4
printer_type	dw	1
;interface	dw	0

tescrow		db	27, 33h, 24		; set graphics line spacing
		db	27, 'K', 224, 1		; start graphics data 480 columns

escrow		db	27, 33h, 24		; set graphics line spacing
		db	27, 'K', 240, 0		; start graphics data 240 columns
;
; NOTE!!! Don't MUCK with the order of the following ESC sequences!
;	'ljg1' expects 'ljxfer' to immediately follow 'ljstart'
;
ljresolution	db	27, '*t75R'		; set laserjet to 75DPI
ljstart		db	27, '*r1A'		; start raster, set position to cursor
ljxfer		db	27, '*b30W'		; transfer graphics data
ljend		db	27, '*rB'		; end raster graphics

ljtxtstart	db	27, '*r1A'
		db	27, '*b60W'

old1coff	dw	?
old1cseg	dw	?

buffer		db	8 dup (?)

attr		db	?

oldbaud		dw	?
oldfram		db	?

;==============================================================================

isr5:
	push	ds
	push	es
	push	si
	push	di
	push	dx
	push	cx
	push	bx
	push	ax
	push	bp

	cld

	mov	ax,50h
	mov	ds,ax

	cmp	byte ptr ds:[0],1	; print screen already active?
	jnz	isr5go			; jif no

isr5exit:
	mov	ax,50h
	mov	ds,ax
	mov	byte ptr ds:[0],0

	pop	bp
	pop	ax
	pop	bx
	pop	cx
	pop	dx
	pop	di
	pop	si
	pop	es
	pop	ds
	iret
;
isr5go:
	mov	byte ptr ds:[0],1	; set "print screen active" flag

	push	cs
	pop	ds
	push	cs
	pop	es

	in	al,21h
	push	ax
	or	al,10h			; mask off the serial interrupt
	out	21h,al

	sti				; enable interrupts

	mov	dx,0e301h
	in	al,dx
	and	al,0fch
	push	ax			; save state of I/O ON bit
	or	al,4			; power it up if it's not already
	out	dx,al

	mov	dx,3fbh
	in	al,dx
	mov	oldfram,al
	or	al,80h
	out	dx,al
	mov	dx,3f8h
	in	ax,dx
	mov	oldbaud,ax
	mov	dx,3fbh
	in	al,dx
	and	al,7fh
	out	dx,al

	mov	ax,baud
	add	ax,3
	cmp	ax,3
	jnz	not300
	dec	ax
not300:
	mov	cl,5
	shl	ax,cl
	or	al,3

	xor	ah,ah
	xor	dx,dx
	int	14h

	cmp	baud,5
	jnz	notbaud5		; jif not 19200

	mov	dx,3fbh
	in	al,dx
	mov	cl,al
	or	al,80h
	out	dx,al
	mov	dx,3f8h			; set 19200 baud divisor by hand
	mov	ax,6
	out	dx,ax
	mov	dx,3fbh
	mov	al,cl
	out	dx,al
notbaud5:
	cmp	printer_type,1		; LaserJet?
	jnz	notlj1			; jif no

	lea	si,ljresolution		; set graphics resolution
	mov	cx,6
	call	lj_send_seq
notlj1:
	mov	si,0b000h
	mov	ds,si			; point to screen memory
	xor	si,si

	mov	dx,0d305h
	in	al,dx
	test	al,1			; in graphics mode?
	jz	isr5text		; jif no
	jmp	isr5graph
;
;==============================================================================

isr5text:
	cmp	cs:printer_type,1	; laserjet?
	jnz	notlaser		; jif no

	jmp	ljtext
notlaser:
	mov	cx,25			; 25 rows
fxt1:
	push	cx

	call	txt_start_row_sequence

	mov	cx,80			; number of chars width of display
fxt2:
	push	cx

	lodsw				; get next char/attribute from screen
	push	si

	call	xlate2font

	mov	bp,6			; output byte count
	call	send_block

	pop	si			; recover screen ptr
	pop	cx			; recover row char counter

	loop	fxt2			; loop til row is done

	call	send_crlf

	pop	cx			; recover row counter

	loop	fxt1			; loop til all rows are done

isr5done:
	push	cs
	pop	ds

	xor	cx,cx
	loop	$

	mov	dx,3fbh
	mov	al,oldfram
	or	al,80h
	out	dx,al
	mov	dx,3f8h
	mov	ax,oldbaud
	out	dx,ax
	mov	dx,3fbh
	mov	al,oldfram
	out	dx,al

	pop	ax
	mov	dx,0e301h
	out	dx,al			; restore state of I/O on bit

	pop	ax			; recover the old serial interrupt mask
	out	21h,al			; restore it

	jmp	isr5exit
;
;==============================================================================

isr5graph:
	cmp	cs:printer_type,1	; laserjet?
	jz	ljgraph			; jif yes

	mov	cx,16			; number of rows of 8 dot-columns
fx1:
	push	cx

	call	start_row_sequence

	mov	cx,30			; number of bytes width of display
fx2:
	push	cx

	lea	di,buffer
	mov	dh,[si]			; get 8 bytes of data
	mov	dl,[si+30]
	mov	ch,[si+60]
	mov	cl,[si+90]
	mov	bh,[si+120]
	mov	bl,[si+150]
	mov	ah,[si+180]
	mov	al,[si+210]

	push	si

	mov	bp,8			; output byte count
	call	send_block

	pop	si
	inc	si

	pop	cx
	loop	fx2			; go send next 8 columns of dots

	call	send_crlf

	add	si,30*7			; skip to start of next row

	pop	cx

	loop	fx1			; go send next row of 8 column dots

	jmp	isr5done
;
;==============================================================================

ljgraph:
	mov	cx,128			; number of rows of raster
ljg1:
	push	cx

	push	si
	mov	cx,11
	lea	si,ljstart		; send start & xfer escape sequences
	call	lj_send_seq
	pop	si

	mov	cx,30
ljg2:
	lodsb
	call	send_byte
	loop	ljg2

	push	si
	mov	cx,4
	lea	si,ljend
	call	lj_send_seq
	pop	si

	pop	cx
	loop	ljg1

	jmp	isr5done
;
;==============================================================================

ljtext:
	mov	cx,25			; 25 rows of chars
ljt1:
	push	cx
	push	si

	mov	bx,0			; raster offset
ljt2:
	push	si

	push	bx
	push	si
	mov	cx,11
	lea	si,ljtxtstart
	call	lj_send_seq
	pop	si
	pop	bx

	mov	cx,20			; 20 sets of 4-chars per line
ljt3:
	push	cx

	mov	cl,0

	call	getfontline
	mov	dh,dl
	call	getfontlineplus
	call	getfontlineplus
	call	getfontlineplus

	pop	cx
	loop	ljt3

	pop	si			; recover screen ptr

	inc	bx
	cmp	bx,8
	jnz	ljt2

	mov	cx,4
	lea	si,ljend
	call	lj_send_seq

	pop	si
	pop	cx			; recover row counter

	add	si,160			; move to next line

	loop	ljt1			; loop til all rows are done

	jmp	isr5done
;
;==============================================================================

getfontlineplus:
	call	getfontline
	mov	al,dh
	mov	dh,dl
	push	dx
	push	cx
	push	bx
	call	send_byte
	pop	bx
	pop	cx
	pop	dx
	ret
;
;==============================================================================

getfontline:
	lodsw				; get next char/attribute
	push	si			; save screen ptr

	mov	cs:attr,ah		; save attribute for later
	mov	si,0fa6eh
	test	al,80h
	jz	ljfxtlow
	mov	si,0b200h
	and	al,7fh
ljfxtlow:
	cbw
	shl	ax,1
	shl	ax,1
	shl	ax,1
	add	si,ax

	push	es
	mov	ax,0f000h
	mov	es,ax			; ES:SI points to font for char
	mov	dl,es:[si+bx]		; get byte of font for char
	pop	es

	pop	si			; recover screen ptr

	and	cs:attr,77h
	jnz	ljfxtnonzero		; jif displayed char

	xor	dl,dl			; else blank font data
	jmp	short ljfxtgoon
ljfxtnonzero:
	cmp	cs:attr,07h		; normal?
	jz	ljfxtgoon		; jif yes
	cmp	cs:attr,70h		; inverse?
	jnz	ljfxtnotinv		; jif no

	xor	dl,0ffh			; else invert
	jmp	short ljfxtgoon
ljfxtnotinv:
	cmp	cs:attr,01h		; underline?
	jnz	ljfxtgoon		; jif no, treat as normal video

	cmp	bx,7			; on bottom line?
	jnz	ljfxtgoon
	or	dl,0ffh			; underline it
ljfxtgoon:
	shl	dl,1
	shl	dl,1
	shl	dx,cl
	inc	cl
	inc	cl
	and	cl,7
	shr	dl,cl
	ret
;
;==============================================================================

xlate2font:
	mov	cs:attr,ah
	mov	si,0fa6eh
	test	al,80h
	jz	fxtlow
	mov	si,0b200h
	and	al,7fh
fxtlow:
	cbw
	shl	ax,1
	shl	ax,1
	shl	ax,1
	add	si,ax
	mov	ax,0f000h
	mov	es,ax			; ES:SI points to font for char

	lea	di,buffer
	mov	dh,es:[si]		; get 8 bytes of data
	mov	dl,es:[si+1]
	mov	ch,es:[si+2]
	mov	cl,es:[si+3]
	mov	bh,es:[si+4]
	mov	bl,es:[si+5]
	mov	ah,es:[si+6]
	mov	al,es:[si+7]

	shl	dx,1
	shl	dx,1
	shl	cx,1
	shl	cx,1
	shl	bx,1
	shl	bx,1
	shl	ax,1
	shl	ax,1

	and	cs:attr,77h
	jnz	fxtnonzero		; jif displayed char

	xor	dx,dx			; else blank font data
	xor	cx,cx
	xor	bx,bx
	xor	ax,ax
	jmp	short fxtgoon
fxtnonzero:
	cmp	cs:attr,07h		; normal?
	jz	fxtgoon			; jif yes
	cmp	cs:attr,70h		; inverse?
	jnz	fxtnotinv		; jif no

	mov	bp,0ffffh
	xor	dx,bp			; else invert
	xor	cx,bp
	xor	bx,bp
	xor	ax,bp
	jmp	short fxtgoon
fxtnotinv:
	cmp	cs:attr,01h		; underline?
	jnz	fxtgoon			; jif no, treat as normal video

	or	al,0ffh			; underline it
fxtgoon:
	ret
;
;==============================================================================

send_block:
	push	es
	push	cs
	pop	es

	push	bp
sbloop:
	shl	dh,1
	rcl	si,1
	shl	dl,1
	rcl	si,1
	shl	ch,1
	rcl	si,1
	shl	cl,1
	rcl	si,1
	shl	bh,1
	rcl	si,1
	shl	bl,1
	rcl	si,1
	shl	ah,1
	rcl	si,1
	shl	al,1
	rcl	si,1

	xchg	ax,si
	stosb				; write out result byte
	xchg	ax,si

	dec	bp
	jnz	sbloop

	pop	cx
	pop	es
;
; !!!WARNING!!! 'send_block' falls through into 'send_block_cx'
;
;==============================================================================

send_block_cx:
	push	ds
	push	si

	push	cs
	pop	ds

	lea	si,buffer
	jmp	short srs_loop
;
;==============================================================================

txt_start_row_sequence:
	push	ds
	push	si

	lea	si,tescrow
	jmp	short srs_com
;
;==============================================================================

start_row_sequence:
	push	ds
	push	si

	lea	si,escrow
srs_com:
	push	cs
	pop	ds

	mov	cx,7
srs_loop:
	lodsb
	call	send_byte

	loop	srs_loop

	pop	si
	pop	ds
	ret
;
;==============================================================================

lj_send_seq:
	push	ds
	push	si

	push	cs
	pop	ds

	jmp	short srs_loop
;
;==============================================================================

send_crlf:
	mov	al,13
	call	send_byte
	mov	al,10
;
; !!!WARNING!!!! 'send_crlf' falls through into 'send_byte'
;
; NOTE!!! 'send_byte' MUST NOT modify CX or SI
;
;==============================================================================

send_byte:
	mov	ah,al			; save byte to send
sb_nowait:
	mov	dx,3fdh
	in	al,dx
	test	al,1			; data incomming?
	jz	sb_loop			; jif no, send byte

	mov	dx,3f8h
	in	al,dx			; get the received byte

	cmp	al,'S'-64		; XOFF?
	jnz	sb_nowait		; jif no
sb_wait2:
	mov	dx,3fdh
sb_wait:
	in	al,dx
	test	al,1			; received another char?
	jz	sb_wait			; jif no

	mov	dx,3f8h
	in	al,dx			; get the received byte

	cmp	al,'Q'-64		; XON?
	jnz	sb_wait2		; jif no
	jmp	sb_nowait
sb_loop:
	in	al,dx
	and	al,60h
	cmp	al,60h
	jnz	sb_loop

	push	cx
	mov	cx,45
	loop	$			; wait a short period to ensure that
	pop	cx			; UART xmitter is REALLY empty.

	mov	al,ah
	mov	dx,3f8h
	out	dx,al

	ret
;
;==============================================================================

isr1c:
	call	take5
	jmp	dword ptr cs:old1coff
;
take5:
	pushf
	cli
	push	ds
	push	si
	xor	si,si
	mov	ds,si
	mov	si,5*4

	mov	word ptr [si],offset isr5
	mov	[si+2],cs

	pop	si
	pop	ds
	popf
	ret

;==============================================================================
; INIT 
;==============================================================================

init:
	lea	dx,fname
	mov	ax,3d00h
	int	21h		; open SETUP.ENV
	jc	nosetup

	push	ax
	mov	bx,ax
	mov	cx,size SETTINGS
	lea	dx,endprog
	mov	ah,3fh		; read contents of SETUP.ENV
	int	21h
	jc	readerr		; jif error

	lea	bx,endprog
	mov	ax,[bx].stngPrBaud
	mov	baud,ax
	mov	ax,[bx].stngPrDriver
	mov	printer_type,ax
;	mov	ax,[bx].stngPrInterface
;	mov	interface,ax
readerr:
	pop	bx
	mov	ah,3eh
	int	21h		; close file
nosetup:
	mov	ax,351ch
	int	21h		; get old INT 1C vector

	mov	old1coff,bx	; save it
	mov	old1cseg,es

	lea	dx,isr1c	; take INT 1C vector
	mov	ax,251ch
	int	21h

	lea	dx,copyr
	mov	ah,9
	int	21h

	lea	dx,init		; TSR, keep BUFSIZE buffer at end of codespace
	add	dx,15
	shr	dx,1
	shr	dx,1
	shr	dx,1
	shr	dx,1
	mov	ax,3100h
	int	21h

copyr	db	"Graphics Print Screen TSR rev 1.00",13,10
	db	"Copyright 1991 Hewlett-Packard Co.",13,10
	db	"All rights reserved",13,10,'$'

endprog:

fname	db	'C:\_DAT\SETUP.ENV',0

cseg	ends
	end	start
